/*  $Id: AObservableRunnable.java 1338 2012-08-25 17:38:30Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2011 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;


/** 
 * Observable Runnable.
 * <p>Let an obser to be notified on runnable state change from READY, 
 * RUNNING, ENDED or EXCEPTION.</p>
 * <p><b>warning:</b> subclasses must implement oRun() method instead of run()
 * as for regular runnables. This class provide a default run() 
 * implatation to manage state transitions and observers notifications.</p>
 * @see Observable observer registration and deregistration rules.
 * @see Observer observer implementation rules.
 */ 
public abstract class AObservableRunnable implements Runnable,
								Observable<AObservableRunnable>
{

	/** 
	 * Create new observable runnable.
	 * Default error handler is App's default error handler. Use 
	 * setErrHandler to provide alternate error handling, before 
	 * running the runnable.
	 */ 
	public AObservableRunnable()
	{
		_state=EState.READY;
		_errHandler=App.get().getErrHandler();
	}
	/** 
	 * Observable runnable entry point.
	 */ 
	public abstract void oRun(); 
	/** 
	 * Runnable implementation.
	 * <b>warning:</b> don't override this method to keep state management
	 * and notification implemented here. Override oRun().
	 */ 
	public void run() 
	{
		_state=EState.RUNNING;
		_obsManager.notify(this);
		try 
		{
			oRun();
			_state=EState.ENDED;
			_obsManager.notify(this);
		}
		catch (Exception ex)
		{
			_exception=ex;
			_state=EState.EXCEPTION;
			_obsManager.notify(this);
			Error err=new Error(""+AObservableRunnable.class+".ex",
							Error.ELevel.FATAL,ex);
			_errHandler.handle(err);
		}
	}
	/** uncatch exception. Stay null unless there is any. */
	private Exception _exception=null;
	/** 
	 * Uncatched exception.
	 * @return when state is EXCEPTION, the uncatched exception cause of
	 * the running failure. Otherwise null.
	 */ 
	public Exception getException()
	{
		return _exception;
	}
	/** current state. Any if READY, RUNNING, ENDED, EXCEPTION */
	private EState _state;
	/** Available runnable states identifications. */
	public enum EState { 
		READY,
		RUNNING,
		ENDED,
		EXCEPTION
	};
	
	/** 
	 * Get current state.
	 * @return 
	 *  - EState.READY : not yet started.<br>
	 *  - EState.RUNNING : started, running in progress.<br>
	 *  - EState.ENDED : running done without exception (errors may
	 *  have been reported to error handler).<br>
	 *  - EState.EXCEPTION : running interrupted by uncatched exception. 
	 *  It is available using getException().<br>
	 *  <b>warning:</b> state may change as soon it is returned if this method
	 *  is called outside the thread running the run() method. Observers
	 *  should not suffer this from their observableChanged() implementation
	 *  that is invoked from the thread running the run() method.
	 */ 
	public EState getState()
	{
		return _state;
	}


	//---------------------------------------------------------------------
	// IObservable implementation
	/** 
	 * Observers manager.
	 * @djin.relation composition
	 */ 
	private ObserverManager<AObservableRunnable> _obsManager=
						new ObserverManager<AObservableRunnable>();
	/**  
	 * @return Observer manager.
	 */ 
	public ObserverManager<AObservableRunnable> getObserverManager()
	{
		return _obsManager;
	}


	//---------------------------------------------------------------------
	// Error management
	/** error handler 
	 * @djin.relation aggregation
	 */ 
	private ErrorHandler _errHandler;
	/** 
	 * @return error handler.
	 */ 
	protected ErrorHandler getErrHandler() 
	{
			return _errHandler;
	}
	/** 
	 * Change error handler. 
	 * Error handler should be defined before running the runnable. If not so,
	 * new error handler may miss some errors.
	 * @param eh new error handler.
	 */ 
	public void setErrorHandler(ErrorHandler eh) 
	{
		_errHandler=eh;
	}
}

